/**
 * Copyright (c) 2015 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * @defgroup tw_sensor_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Sensor Example main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 */

#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"


#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

/* Testbed Configuration Struct Placeholder */
#define PRINTF(...) NRF_LOG_INFO(__VA_ARGS__)
#include "testbed.h"
volatile config_t __attribute((section (".testbedConfigSection"))) cfg ={0};
//volatile config_t __attribute((section (".testbedConfigSection"))) cfg ={1,{{1,{208},{216},8,0,0,0,500,30000}}};

/* Custom Testbed Configuration Struct Placeholder */
#include "custom.h"
volatile custom_config_t __attribute((section (".customConfigSection"))) custom_cfg={0};


/* Common addresses definition for eeprom. */
#define EEPROM_ADDR          (0xA0U >> 1)

/* Testbed I2C Pin definition. */
#define TWI0_CONFIG_SCL          3
#define TWI0_CONFIG_SDA          4

#define PIN_IN NRF_GPIO_PIN_MAP(1,2)

/* TWI instance ID. */
#define TWI_INSTANCE_ID     0

/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);
static volatile bool m_xfer_done = false;

/**
 * @brief TWI events handler.
 */
void twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
    m_xfer_done = true;
    switch (p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            break;
        default:
            break;
    }
}

/**
 * @brief I2C initialization.
 */
void twi_init (void)
{
    ret_code_t err_code;

    const nrf_drv_twi_config_t twi_eeprom_config = {
       .scl                = TWI0_CONFIG_SCL,
       .sda                = TWI0_CONFIG_SDA,
       .frequency          = NRF_DRV_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = false
    };

    err_code = nrf_drv_twi_init(&m_twi, &twi_eeprom_config, twi_handler, NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_twi_enable(&m_twi);
}

/**
 * @brief Function for reading data from the EEPROM.
 */
void eeprom_readBytes(uint16_t eeprom_address, unsigned char* val, uint8_t len){
	NRF_LOG_INFO("Reading");
	unsigned char eeprom_by_address[2];
    ret_code_t err_code;
        
	memset(val,0,cfg.p[0].msg_length);

	eeprom_by_address[1]	= eeprom_address;
	eeprom_by_address[0] = (unsigned char)(eeprom_address << 8);
	// setting the start address
    do{
        m_xfer_done = false;
        err_code = nrf_drv_twi_tx(&m_twi, EEPROM_ADDR, eeprom_by_address, 2, true);
        if(NRF_SUCCESS != err_code)
            continue;
        while(!m_xfer_done);
        m_xfer_done = false;
        err_code = nrf_drv_twi_rx(&m_twi, EEPROM_ADDR, val, len);
    }while(NRF_SUCCESS != err_code);
    while(!m_xfer_done);
    NRF_LOG_INFO("Done");
}

/**
 * @brief Function for writing data to the EEPROM.
 */
void eeprom_writeBytes(uint16_t eeprom_address, unsigned char* val, uint8_t len){
	NRF_LOG_INFO("Writing");
	unsigned char chunk[257];
    ret_code_t err_code;
        
	chunk[1]	= eeprom_address;
	chunk[0] = (unsigned char)(eeprom_address << 8);
	for(int i=0;i<len;i++)
        chunk[i+2]=val[i];
    
    do{
        m_xfer_done = false;
        err_code = nrf_drv_twi_tx(&m_twi, EEPROM_ADDR, chunk, len+2, false);
        if(NRF_SUCCESS != err_code)
            continue;
        while(!m_xfer_done);
    }while(NRF_SUCCESS != err_code);
    while(!m_xfer_done);
    NRF_LOG_INFO("Done");
}

/**
 * @brief Function for comparing the content of the EEPROM against a known Value.
 */
void check_string(const char* val){
    int i;
    NRF_LOG_INFO("New Message");
	for(i=0;i<cfg.p[0].msg_length;i++)
	{
        NRF_LOG_RAW_INFO("%02X ",val[i]);
	}
    NRF_LOG_RAW_INFO("\n");
    
    /* Do Processing */
}

static uint8_t buffer[255]={};

/**
 * @brief GPIO pin change handler.
 */
void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    NRF_LOG_INFO("Signal received");
    eeprom_readBytes(0,buffer,cfg.p[0].msg_length);
    check_string((const char*)buffer);
}

/**
 * @brief Function for initializing the boards GPIOs
 */
void gpio_init(){
     ret_code_t err_code;
        
    //GPIO block monitored by the testbed (P1.01-P1.08)
	for(uint8_t pin = 1; pin < 9; pin++)
	{
		nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(1,pin));
		nrf_gpio_pin_clear(NRF_GPIO_PIN_MAP(1,pin));
	}
    
    //EEPROM data is signaled on pin P1.02
    nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(1,2));
    
    //LED for local debugging
	nrf_gpio_cfg_output(NRF_GPIO_PIN_MAP(0,16));
	nrf_gpio_pin_set(NRF_GPIO_PIN_MAP(0,16));
    
    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);
    //in_config.pull = NRF_GPIO_PIN_PULLUP; //drive by the testbed only if selected as output

    err_code = nrf_drv_gpiote_in_init(PIN_IN, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(PIN_IN, true);
}

/**
 * @brief Function for main application entry.
 */
int main(void)
{
    uint8_t motd[255];
    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));
    NRF_LOG_DEFAULT_BACKENDS_INIT();

    NRF_LOG_INFO("\r\nRFDC_1 EEPROM example started.");
    NRF_LOG_INFO("Todays Magic number is %u",custom_cfg.magic);
    print_testbed_config((config_t*)&cfg);
    NRF_LOG_FLUSH();
    gpio_init();
    twi_init();
    
    memset(motd,custom_cfg.magic,cfg.p[0].msg_length);
    
    eeprom_writeBytes(0,motd,cfg.p[0].msg_length);
    nrf_delay_ms(100);
    eeprom_readBytes(0,buffer,cfg.p[0].msg_length);
    check_string((const char*)buffer);

    while (true)
    {
        // Do nothing.
        NRF_LOG_PROCESS();
    }
}

/** @} */
